@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
-
-from urlparse import urlparse, parse_qsl
+import warnings
+from urllib.parse import parse_qsl, urlparse
 
 import gmusicapi
 from gmusicapi.clients.shared import _Base
+from gmusicapi.exceptions import GmusicapiWarning
 from gmusicapi.protocol import webclient
 from gmusicapi.utils import utils
 import gmusicapi.session
@@ -22,8 +22,6 @@ class Webclient(_Base):
 
     The following methods are *not* deprecated:
 
-        * :func:`create_playlist`
-        * :func:`get_registered_devices`
         * :func:`get_shared_playlist_info`
         * :func:`get_song_download_info`
         * :func:`get_stream_urls`
@@ -35,21 +33,30 @@ class Webclient(_Base):
     _session_class = gmusicapi.session.Webclient
 
     def __init__(self, debug_logging=True, validate=True, verify_ssl=True):
-        super(Webclient, self).__init__(self.__class__.__name__,
-                                        debug_logging,
-                                        validate,
-                                        verify_ssl)
+        warnings.warn(
+            "Webclient functionality is not tested nor well supported. "
+            "Use Mobileclient or Musicmanager if possible.",
+            GmusicapiWarning
+        )
+
+        super().__init__(self.__class__.__name__,
+                         debug_logging,
+                         validate,
+                         verify_ssl)
 
     def login(self, email, password):
         """Authenticates the webclient.
         Returns ``True`` on success, ``False`` on failure.
 
         :param email: eg ``'test@gmail.com'`` or just ``'test'``.
-        :param password: password or app-specific password for 2-factor users.
+        :param password: the account's password.
           This is not stored locally, and is sent securely over SSL.
+          App-specific passwords are not supported on the webclient.
 
-        Users of two-factor authentication will need to set an application-specific password
-        to log in.
+        Users who don't use two-factor auth will likely need to enable
+        `less secure login <https://www.google.com/settings/security/lesssecureapps>`__.
+        If this is needed, a warning will be logged during login (which will print to stderr
+        in the default logging configuration).
         """
 
         if not self.session.login(email, password):
@@ -61,19 +68,7 @@ class Webclient(_Base):
         return True
 
     def logout(self):
-        return super(Webclient, self).logout()
-
-    def create_playlist(self, name, description, public=False):
-        """
-        Creates a playlist and returns its id.
-
-        :param name: the name of the playlist.
-        :param description: the description of the playlist.
-        :param public: if True and the user has All Access, create a shared playlist.
-        """
-        res = self._make_call(webclient.CreatePlaylist, name, description, public)
-
-        return res[1][0]
+        return super().logout()
 
     def get_shared_playlist_info(self, share_token):
         """
@@ -91,56 +86,12 @@ class Webclient(_Base):
         md = res[1][1]
 
         return {
-            u'author': md[8],
-            u'description': md[7],
-            u'num_tracks': num_tracks,
-            u'title': md[1],
+            'author': md[8],
+            'description': md[7],
+            'num_tracks': num_tracks,
+            'title': md[1],
         }
 
-    def get_registered_devices(self):
-        """
-        Returns a list of dictionaries representing devices associated with the account.
-
-        Performing the :class:`Musicmanager` OAuth flow will register a device
-        of type ``'DESKTOP_APP'``.
-
-        Installing the Android Google Music app and logging into it will
-        register a device of type ``'PHONE'``, which is required for streaming with
-        the :class:`Mobileclient`.
-
-        Here is an example response::
-
-            [
-              {
-                u'date': 1367470393588,           # utc-millisecond
-                u'id':   u'AA:BB:CC:11:22:33',
-                u'lastUsedMs': 1394138679694,
-                u'name': u'my-hostname',
-                u'type': u'DESKTOP_APP'
-              },
-              {
-                u'carrier':      u'Google',
-                u'date':         1344808742774,
-                u'id':           u'0x00112233aabbccdd',  # remove 0x when streaming
-                u'manufacturer': u'Asus',
-                u'model':        u'Nexus 7',
-                u'name':         u'',
-                u'type':         u'PHONE'
-              },
-              {
-                u'date': 1394133624308,
-                u'id': u'ios:01234567-0123-0123-0123-0123456789AB',
-                u'lastUsedMs': 1394138679694,
-                u'type': u'IOS'
-              }
-            ]
-
-        """
-
-        #TODO sessionid stuff
-        res = self._make_call(webclient.GetSettings, '')
-        return res['settings']['devices']
-
     @utils.enforce_id_param
     def get_song_download_info(self, song_id):
         """Returns a tuple: ``('<url>', <download count>)``.
@@ -156,7 +107,7 @@ class Webclient(_Base):
         the count is incremented when ``url`` is retrieved.
         """
 
-        #TODO the protocol expects a list of songs - could extend with accept_singleton
+        # TODO the protocol expects a list of songs - could extend with accept_singleton
         info = self._make_call(webclient.GetDownloadInfo, [song_id])
         url = info.get('url')
 
@@ -211,13 +162,13 @@ class Webclient(_Base):
 
           There are three possible values for this argument:
               * None: (default) send header; fix response locally on problems
-              * True: send header; raise IOError on problems
+              * True: send header; raise OSError on problems
               * False: do not send header
         """
 
         urls = self.get_stream_urls(song_id)
 
-        #TODO shouldn't session.send be used throughout?
+        # TODO shouldn't session.send be used throughout?
 
         if len(urls) == 1:
             return self.session._rsession.get(urls[0]).content
@@ -230,7 +181,7 @@ class Webclient(_Base):
                        for key, val in parse_qsl(urlparse(url)[4])
                        if key == 'range']
 
-        stream_pieces = []
+        stream_pieces = bytearray()
         prev_end = 0
         headers = None
 
@@ -241,24 +192,24 @@ class Webclient(_Base):
             audio = self.session._rsession.get(url, headers=headers).content
 
             if end - prev_end != len(audio) - 1:
-                #content length is not in the right range
+                # content length is not in the right range
 
                 if use_range_header:
                     # the user didn't want automatic response fixup
-                    raise IOError('use_range_header is True but the response'
+                    raise OSError('use_range_header is True but the response'
                                   ' was not the correct content length.'
                                   ' This might be caused by a (poorly-written) http proxy.')
 
                 # trim to the proper range
                 audio = audio[prev_end - start:]
 
-            stream_pieces.append(audio)
+            stream_pieces.extend(audio)
 
             prev_end = end + 1
 
-        return ''.join(stream_pieces)
+        return bytes(stream_pieces)
 
-    @utils.accept_singleton(basestring)
+    @utils.accept_singleton(str)
     @utils.enforce_ids_param
     @utils.empty_arg_shortcircuit
     def report_incorrect_match(self, song_ids):
@@ -269,7 +220,7 @@ class Webclient(_Base):
 
         Note that if you uploaded a song through gmusicapi, it won't be reuploaded
         automatically - this currently only works for songs uploaded with the Music Manager.
-        See issue `#89 <https://github.com/simon-weber/Unofficial-Google-Music-API/issues/89>`__.
+        See issue `#89 <https://github.com/simon-weber/gmusicapi/issues/89>`__.
 
         This should only be used on matched tracks (``song['type'] == 6``).
         """
@@ -278,7 +229,7 @@ class Webclient(_Base):
 
         return song_ids
 
-    @utils.accept_singleton(basestring)
+    @utils.accept_singleton(str)
     @utils.enforce_ids_param
     @utils.empty_arg_shortcircuit
     def upload_album_art(self, song_ids, image_filepath):
@@ -302,9 +253,87 @@ class Webclient(_Base):
 
         return url
 
+    @utils.accept_singleton(dict)
+    @utils.empty_arg_shortcircuit
+    def change_song_metadata(self, songs):
+        """Changes metadata of songs.
+
+        Returns a list of the song ids changed.
+
+        :param songs: a list of song dictionaries, each dictionary must contain valid song 'id'
+
+        The following fields are supported: title, album, albumArtist, artist
+        """
+
+        self._make_call(webclient.ChangeSongMetadata, songs)
+
+        return list(song['id'] for song in songs)
+
     # deprecated methods follow:
 
-    @utils.accept_singleton(basestring)
+    @utils.deprecated('prefer Mobileclient.create_playlist')
+    def create_playlist(self, name, description=None, public=False):
+        """
+        Creates a playlist and returns its id.
+
+        :param name: the name of the playlist.
+        :param description: (optional) the description of the playlist.
+        :param public: if True and the user has All Access, create a shared playlist.
+        """
+        res = self._make_call(webclient.CreatePlaylist, name, description, public)
+
+        return res[1][0]
+
+    @utils.deprecated('prefer Mobileclient.get_registered_devices')
+    def get_registered_devices(self):
+        """
+        Returns a list of dictionaries representing devices associated with the account.
+
+        Performing the :class:`Musicmanager` OAuth flow will register a device
+        of type 1.
+
+        Installing the Google Music app on an android or ios device
+        and logging into it will register a device of type 2 or 3,
+        which is used for streaming with the :class:`Mobileclient`.
+
+        Here is an example response::
+
+            [
+              {
+                u'deviceType': 1,  # laptop/desktop
+                u'id': u'00:11:22:33:AA:BB',
+                u'lastAccessedFormatted': u'May 24, 2015',
+                u'lastAccessedTimeMillis': 1432468588200,  # utc-millisecond
+                u'lastEventTimeMillis': 1434211605335,
+                u'name': u'my computer'},
+              },
+              {
+                u'deviceType': 2,  # android device
+                u'carrier': u'Google',
+                u'id': u'0x00112233aabbccdd',  # remove 0x when streaming
+                u'lastAccessedFormatted': u'September 19, 2015',
+                u'lastAccessedTimeMillis': 1442706069906,
+                u'lastEventTimeMillis': 1435271137193,
+                u'manufacturer': u'Asus',
+                u'model': u'Nexus 7',
+                u'name': u'my nexus 7'
+              },
+              {
+                u'deviceType': 3,  # ios device
+                u'id': u'ios:01234567-0123-0123-0123-0123456789AB',
+                u'lastAccessedFormatted': u'June 25, 2015',
+                u'lastAccessedTimeMillis': 1435271588780,
+                u'lastEventTimeMillis': 1435271442417,
+                u'name': u'my iphone'
+              }
+            ]
+        """
+
+        # TODO sessionid stuff
+        res = self._make_call(webclient.GetSettings, '')
+        return res['settings']['uploadDevice']
+
+    @utils.accept_singleton(str)
     @utils.enforce_ids_param
     @utils.empty_arg_shortcircuit
     @utils.deprecated('prefer Mobileclient.delete_songs')
@@ -320,7 +349,7 @@ class Webclient(_Base):
 
         return res['deleteIds']
 
-    @utils.accept_singleton(basestring, 2)
+    @utils.accept_singleton(str, 2)
     @utils.enforce_ids_param(2)
     @utils.enforce_id_param
     @utils.empty_arg_shortcircuit(position=2)
@@ -342,7 +371,7 @@ class Webclient(_Base):
 
         return [(e['songId'], e['playlistEntryId']) for e in new_entries]
 
-    @utils.accept_singleton(basestring, 2)
+    @utils.accept_singleton(str, 2)
     @utils.enforce_ids_param(2)
     @utils.enforce_id_param
     @utils.empty_arg_shortcircuit(position=2)
@@ -368,14 +397,14 @@ class Webclient(_Base):
                          if t["id"] in sid_set]
 
         if matching_eids:
-            #Call returns "sid_eid" strings.
+            # Call returns "sid_eid" strings.
             sid_eids = self._remove_entries_from_playlist(playlist_id,
                                                           matching_eids)
             return [s.split("_") for s in sid_eids]
         else:
             return []
 
-    @utils.accept_singleton(basestring, 2)
+    @utils.accept_singleton(str, 2)
     @utils.empty_arg_shortcircuit(position=2)
     def _remove_entries_from_playlist(self, playlist_id, entry_ids_to_remove):
         """Removes entries from a playlist. Returns a list of removed "sid_eid" strings.
@@ -384,7 +413,7 @@ class Webclient(_Base):
         :param entry_ids: a list of entry ids, or a single entry id.
         """
 
-        #GM requires the song ids in the call as well; find them.
+        # GM requires the song ids in the call as well; find them.
         playlist_tracks = self.get_playlist_songs(playlist_id)
         remove_eid_set = set(entry_ids_to_remove)
 
@@ -397,8 +426,8 @@ class Webclient(_Base):
             self.logger.warning("when removing, %d entry ids could not be found in playlist id %s",
                                 num_not_found, playlist_id)
 
-        #Unzip the pairs.
-        sids, eids = zip(*e_s_id_pairs)
+        # Unzip the pairs.
+        sids, eids = list(zip(*e_s_id_pairs))
 
         res = self._make_call(webclient.DeleteSongs, sids, playlist_id, eids)
 
