@@ -1,20 +1,18 @@
-# -*- coding: utf-8 -*-
-
 """Calls made by the Music Manager (related to uploading)."""
 
 import base64
-from collections import namedtuple
 import hashlib
+import itertools
 import os
-import sys
+import shutil
+from tempfile import NamedTemporaryFile
 
 import dateutil.parser
 from decorator import decorator
 from google.protobuf.message import DecodeError
 import mutagen
-from oauth2client.client import OAuth2Credentials
 
-from gmusicapi.compat import json
+import json
 from gmusicapi.exceptions import CallFailure
 from gmusicapi.protocol import upload_pb2, locker_pb2, download_pb2
 from gmusicapi.protocol.shared import Call, ParseException, authtypes
@@ -23,41 +21,8 @@ from gmusicapi.utils import utils
 log = utils.DynamicClientLogger(__name__)
 
 
-#This url has SSL issues, hence the static_verify=False.
 _android_url = 'https://android.clients.google.com/upsj/'
 
-OAuthInfo = namedtuple('OAuthInfo', 'client_id client_secret scope redirect')
-oauth = OAuthInfo(
-    '652850857958.apps.googleusercontent.com',
-    'ji1rklciNp2bfsFJnEH_i6al',
-    'https://www.googleapis.com/auth/musicmanager',
-    'urn:ietf:wg:oauth:2.0:oob'
-)
-
-
-def credentials_from_refresh_token(token):
-    # why doesn't Google provide this!?
-
-    cred_json = {"_module": "oauth2client.client",
-                 "token_expiry": "2000-01-01T00:13:37Z",  # to refresh now
-                 "access_token": 'bogus',
-                 "token_uri": "https://accounts.google.com/o/oauth2/token",
-                 "invalid": False,
-                 "token_response": {
-                     "access_token": 'bogus',
-                     "token_type": "Bearer",
-                     "expires_in": 3600,
-                     "refresh_token": token},
-                 "client_id": oauth.client_id,
-                 "id_token": None,
-                 "client_secret": oauth.client_secret,
-                 "revoke_uri": "https://accounts.google.com/o/oauth2/revoke",
-                 "_class": "OAuth2Credentials",
-                 "refresh_token": token,
-                 "user_agent": None}
-
-    return OAuth2Credentials.new_from_json(json.dumps(cred_json))
-
 
 @decorator
 def pb(f, *args, **kwargs):
@@ -87,9 +52,7 @@ class MmCall(Call):
         try:
             res_msg.ParseFromString(response.content)
         except DecodeError as e:
-            trace = sys.exc_info()[2]
-            raise ParseException(str(e)), None, trace
-            pass
+            raise ParseException(str(e)) from e
 
         return res_msg
 
@@ -98,11 +61,27 @@ class MmCall(Call):
         return Call._filter_proto(msg)
 
 
+class GetClientState(MmCall):
+    static_url = _android_url + 'clientstate'
+
+    @classmethod
+    @pb
+    def dynamic_data(cls, uploader_id):
+        """
+        :param uploader_id: MM uses host MAC address
+        """
+
+        req_msg = upload_pb2.ClientStateRequest()
+
+        req_msg.uploader_id = uploader_id
+
+        return req_msg
+
+
 class AuthenticateUploader(MmCall):
     """Sent to auth, reauth, or register our upload client."""
 
     static_url = _android_url + 'upauth'
-    static_verify = False
 
     @classmethod
     def check_success(cls, response, msg):
@@ -134,31 +113,45 @@ class AuthenticateUploader(MmCall):
 
 class UploadMetadata(MmCall):
     static_url = _android_url + 'metadata'
-    static_verify = False
 
     static_params = {'version': 1}
 
     @staticmethod
     def get_track_clientid(filepath):
-        #The id is a 22 char hash of the file. It is found by:
+        # The id is a 22 char hash of the file. It is found by:
         # stripping tags
         # getting an md5 sum
         # converting sum to base64
         # removing trailing ===
 
-        #My implementation is _not_ the same hash the music manager will send;
-        # they strip tags first. But files are differentiated across accounts,
-        # so this shouldn't cause problems.
-
-        #This will attempt to reupload files if their tags change.
-
         m = hashlib.md5()
-        with open(filepath, 'rb') as f:
-            m.update(f.read())
+
+        try:
+            ext = os.path.splitext(filepath)[1]
+
+            # delete=False is needed because the NamedTemporaryFile
+            # can't be opened by name a second time on Windows otherwise.
+            with NamedTemporaryFile(suffix=ext, delete=False) as temp:
+                shutil.copy(filepath, temp.name)
+
+                audio = mutagen.File(temp.name, easy=True)
+                audio.delete()
+                audio.save()
+
+                while True:
+                    data = temp.read(65536)
+                    if not data:
+                        break
+                    m.update(data)
+        finally:
+            try:
+                os.remove(temp.name)
+            except OSError:
+                log.exception("Could not remove temporary file %r", temp.name)
 
         return base64.encodestring(m.digest())[:-3]
 
-    #these collections define how locker_pb2.Track fields align to mutagen's.
+    # these collections define how locker_pb2.Track fields align to mutagen's.
     shared_fields = ('album', 'artist', 'composer', 'genre')
     field_map = {  # mutagen: Track
         'albumartist': 'album_artist',
@@ -175,54 +168,70 @@ class UploadMetadata(MmCall):
         On problems, raise ValueError."""
         track = locker_pb2.Track()
 
+        # The track protobuf message supports an additional metadata list field.
+        # ALBUM_ART_HASH has been observed being sent in this field so far.
+        # Append locker_pb2.AdditionalMetadata objects to additional_metadata.
+        # AdditionalMetadata objects consist of two fields, 'tag_name' and 'value'.
+        additional_metadata = []
+
         track.client_id = cls.get_track_clientid(filepath)
 
+        audio = mutagen.File(filepath, easy=True)
+
+        if audio is None:
+            raise ValueError("could not open to read metadata")
+        elif isinstance(audio, mutagen.asf.ASF):
+            # WMA entries store more info than just the value.
+            # Monkeypatch in a dict {key: value} to keep interface the same for all filetypes.
+            asf_dict = {k: [ve.value for ve in v] for (k, v) in audio.tags.as_dict().items()}
+            audio.tags = asf_dict
+
         extension = os.path.splitext(filepath)[1].upper()
+
+        if isinstance(extension, bytes):
+            extension = extension.decode('utf8')
+
         if extension:
-            #Trim leading period if it exists (ie extension not empty).
+            # Trim leading period if it exists (ie extension not empty).
             extension = extension[1:]
 
+        if isinstance(audio, mutagen.mp4.MP4) and (
+                audio.info.codec == 'alac' or audio.info.codec_description == 'ALAC'):
+            extension = 'ALAC'
+        elif isinstance(audio, mutagen.mp4.MP4) and audio.info.codec_description.startswith('AAC'):
+            extension = 'AAC'
+
         if extension.upper() == 'M4B':
-            #M4B are supported by the music manager, and transcoded like normal.
+            # M4B are supported by the music manager, and transcoded like normal.
             extension = 'M4A'
 
         if not hasattr(locker_pb2.Track, extension):
-            raise ValueError("unsupported filetype")
+            raise ValueError("unsupported filetype: {0} for file {1}".format(extension, filepath))
 
         track.original_content_type = getattr(locker_pb2.Track, extension)
 
         track.estimated_size = os.path.getsize(filepath)
         track.last_modified_timestamp = int(os.path.getmtime(filepath))
 
-        #These are typically zeroed in my examples.
+        # These are typically zeroed in my examples.
         track.play_count = 0
         track.client_date_added = 0
         track.recent_timestamp = 0
         track.rating = locker_pb2.Track.NOT_RATED  # star rating
 
-        #Populate information about the encoding.
-        audio = mutagen.File(filepath, easy=True)
-        if audio is None:
-            raise ValueError("could not open to read metadata")
-        elif isinstance(audio, mutagen.asf.ASF):
-            #WMA entries store more info than just the value.
-            #Monkeypatch in a dict {key: value} to keep interface the same for all filetypes.
-            asf_dict = dict((k, [ve.value for ve in v]) for (k, v) in audio.tags.as_dict().items())
-            audio.tags = asf_dict
-
         track.duration_millis = int(audio.info.length * 1000)
 
         try:
-            bitrate = int(audio.info.bitrate / 1000)
+            bitrate = audio.info.bitrate // 1000
         except AttributeError:
-            #mutagen doesn't provide bitrate for some lossless formats (eg FLAC), so
+            # mutagen doesn't provide bitrate for some lossless formats (eg FLAC), so
             # provide an estimation instead. This shouldn't matter too much;
             # the bitrate will always be > 320, which is the highest scan and match quality.
-            bitrate = (track.estimated_size * 8) / track.duration_millis
+            bitrate = (track.estimated_size * 8) // track.duration_millis
 
         track.original_bit_rate = bitrate
 
-        #Populate metadata.
+        # Populate metadata.
 
         def track_set(field_name, val, msg=track):
             """Returns result of utils.pb_set and logs on failures.
@@ -230,12 +239,12 @@ class UploadMetadata(MmCall):
             success = utils.pb_set(msg, field_name, val)
 
             if not success:
-                log.info("could not pb_set track.%s = %r for '%s'", field_name, val, filepath)
+                log.info("could not pb_set track.%s = %r for '%r'", field_name, val, filepath)
 
             return success
 
-        #Title is required.
-        #If it's not in the metadata, the filename will be used.
+        # Title is required.
+        # If it's not in the metadata, the filename will be used.
         if "title" in audio:
             title = audio['title'][0]
             if isinstance(title, mutagen.asf.ASFUnicodeAttribute):
@@ -243,7 +252,7 @@ class UploadMetadata(MmCall):
 
             track_set('title', title)
         else:
-            #Assume ascii or unicode.
+            # Assume ascii or unicode.
             track.title = os.path.basename(filepath)
 
         if "date" in audio:
@@ -253,23 +262,23 @@ class UploadMetadata(MmCall):
             except (ValueError, TypeError) as e:
                 # TypeError provides compatibility with:
                 #  https://bugs.launchpad.net/dateutil/+bug/1247643
-                log.warning("could not parse date md for '%s': (%s)", filepath, e)
+                log.warning("could not parse date md for '%r': (%s)", filepath, e)
             else:
                 track_set('year', datetime.year)
 
         for null_field in ['artist', 'album']:
             # If these fields aren't provided, they'll render as "undefined" in the web interface;
-            # see https://github.com/simon-weber/Unofficial-Google-Music-API/issues/236.
+            # see https://github.com/simon-weber/gmusicapi/issues/236.
             # Defaulting them to an empty string fixes this.
             if null_field not in audio:
                 track_set(null_field, '')
 
-        #Mass-populate the rest of the simple fields.
-        #Merge shared and unshared fields into {mutagen: Track}.
+        # Mass-populate the rest of the simple fields.
+        # Merge shared and unshared fields into {mutagen: Track}.
         fields = dict(
-            dict((shared, shared) for shared in cls.shared_fields).items() +
-            cls.field_map.items()
-        )
+            itertools.chain(
+                ((shared, shared) for shared in cls.shared_fields),
+                cls.field_map.items()))
 
         for mutagen_f, track_f in fields.items():
             if mutagen_f in audio:
@@ -283,6 +292,9 @@ class UploadMetadata(MmCall):
                 if len(numstrs) == 2 and numstrs[1]:
                     track_set(track_total_f, numstrs[1])
 
+        if additional_metadata:
+            track.track_extras.additional_metadata.extend(additional_metadata)
+
         return track
 
     @classmethod
@@ -307,9 +319,8 @@ class UploadMetadata(MmCall):
 
 
 class GetUploadJobs(MmCall):
-    #TODO
+    # TODO
     static_url = _android_url + 'getjobs'
-    static_verify = False
 
     static_params = {'version': 1}
 
@@ -336,7 +347,7 @@ class GetUploadSession(MmCall):
     This is a json call, and doesn't share much with the other calls."""
 
     static_method = 'POST'
-    static_url = 'https://uploadsj.clients.google.com/uploadsj/rupio'
+    static_url = 'https://uploadsj.clients.google.com/uploadsj/scottyagent'
 
     @classmethod
     def parse_response(cls, response):
@@ -350,8 +361,12 @@ class GetUploadSession(MmCall):
     def dynamic_data(uploader_id, num_already_uploaded,
                      track, filepath, server_id, do_not_rematch=False):
         """track is a locker_pb2.Track, and the server_id is from a metadata upload."""
-        #small info goes inline, big things get their own external PUT.
-        #still not sure as to thresholds - I've seen big album art go inline.
+        # small info goes inline, big things get their own external PUT.
+        # still not sure as to thresholds - I've seen big album art go inline.
+
+        if isinstance(filepath, bytes):
+            filepath = filepath.decode('utf8')
+
         inlined = {
             "title": "jumper-uploader-title-42",
             "ClientId": track.client_id,
@@ -374,8 +389,8 @@ class GetUploadSession(MmCall):
                             "filename": os.path.basename(filepath),
                             "name": os.path.abspath(filepath),
                             "put": {},
-                            #used to use this; don't see it in examples
-                            #"size": track.estimated_size,
+                            # used to use this; don't see it in examples
+                            # "size": track.estimated_size,
                         }
                     }
                 ]
@@ -383,10 +398,10 @@ class GetUploadSession(MmCall):
             "protocolVersion": "0.8"
         }
 
-        #Insert the inline info.
+        # Insert the inline info.
         for key in inlined:
             payload = inlined[key]
-            if not isinstance(payload, basestring):
+            if not isinstance(payload, str):
                 payload = str(payload)
 
             message['createSessionRequest']['fields'].append(
@@ -409,10 +424,14 @@ class GetUploadSession(MmCall):
             return (True, None)
 
         if 'errorMessage' in res:
-            #This terribly nested structure is Google's doing.
-            error_code = (res['errorMessage']['additionalInfo']
-                          ['uploader_service.GoogleRupioAdditionalInfo']
-                          ['completionInfo']['customerSpecificInfo']['ResponseCode'])
+            try:
+                # This terribly nested structure is Google's doing.
+                error_code = (res['errorMessage']['additionalInfo']
+                              ['uploader_service.GoogleRupioAdditionalInfo']['completionInfo']
+                              ['customerSpecificInfo']['ResponseCode'])
+            except KeyError:
+                # The returned nested structure is not as expected: cannot get Response Code
+                error_code = None
 
             got_session = False
 
@@ -420,7 +439,7 @@ class GetUploadSession(MmCall):
                 should_retry = True
                 reason = 'upload servers still syncing'
 
-            #TODO unsure about these codes
+            # TODO unsure about these codes
             elif error_code == 200:
                 should_retry = False
                 reason = 'this song is already uploaded'
@@ -440,7 +459,7 @@ class GetUploadSession(MmCall):
 
 class UploadFile(MmCall):
     """Called after getting a session to actually upload a file."""
-    #TODO recent protocols use multipart encoding
+    # TODO recent protocols use multipart encoding
 
     static_method = 'PUT'
 
@@ -458,7 +477,7 @@ class UploadFile(MmCall):
 
     @staticmethod
     def dynamic_url(session_url, content_type, audio):
-        #this actually includes params, but easier to pass them straight through
+        # this actually includes params, but easier to pass them straight through
         return session_url
 
     @staticmethod
@@ -473,12 +492,11 @@ class ProvideSample(MmCall):
     static_method = 'POST'
     static_params = {'version': 1}
     static_url = _android_url + 'sample'
-    static_verify = False
 
     @staticmethod
     @pb
     def dynamic_data(filepath, server_challenge, track, uploader_id, mock_sample=None):
-        """Raise IOError on transcoding problems, or ValueError for invalid input.
+        """Raise OSError on transcoding problems, or ValueError for invalid input.
 
         :param mock_sample: if provided, will be sent in place of a proper sample
 
@@ -494,17 +512,17 @@ class ProvideSample(MmCall):
         sample_spec = server_challenge.challenge_info  # convenience
 
         if mock_sample is None:
-            #The sample is simply a small (usually 15 second) clip of the song,
+            # The sample is simply a small (usually 15 second) clip of the song,
             # transcoded into 128kbs mp3. The server dictates where the cut should be made.
             sample_msg.sample = utils.transcode_to_mp3(
                 filepath, quality='128k',
-                slice_start=sample_spec.start_millis / 1000,
-                slice_duration=sample_spec.duration_millis / 1000
+                slice_start=sample_spec.start_millis // 1000,
+                slice_duration=sample_spec.duration_millis // 1000
             )
         else:
             sample_msg.sample = mock_sample
 
-        #You can provide multiple samples; I just provide one at a time.
+        # You can provide multiple samples; I just provide one at a time.
         msg.track_sample.extend([sample_msg])
 
         return msg
@@ -519,8 +537,7 @@ class UpdateUploadState(MmCall):
 
     static_method = 'POST'
     static_params = {'version': 1}
-    static_url = _android_url + 'sample'
-    static_verify = False
+    static_url = _android_url + 'uploadstate'
 
     @staticmethod
     @pb
@@ -551,7 +568,6 @@ class CancelUploadJobs(MmCall):
 
     static_method = 'POST'
     static_url = _android_url + 'deleteuploadrequested'
-    static_verify = False
 
     @staticmethod
     @pb
@@ -626,7 +642,7 @@ class ListTracks(MmCall):
                 ), cls.__name__
             )
 
-    #TODO
+    # TODO
     @staticmethod
     def filter_response(msg):
         """Only log a summary."""
