@@ -1,14 +1,10 @@
-# -*- coding: utf-8 -*-
-
 """Definitions shared by multiple clients."""
 
 from collections import namedtuple
-import sys
 
 from google.protobuf.descriptor import FieldDescriptor
 
-import gmusicapi
-from gmusicapi.compat import json
+import json
 from gmusicapi.exceptions import (
     CallFailure, ParseException, ValidationException,
 )
@@ -18,13 +14,14 @@ import requests
 
 log = utils.DynamicClientLogger(__name__)
 
-_auth_names = ('xt', 'sso', 'oauth')
+_auth_names = ('xt', 'sso', 'oauth', 'gpsoauth',)
 
 """
   AuthTypes has fields for each type of auth, each of which store a bool:
     xt: webclient xsrf param/cookie
     sso: webclient Authorization header
-    oauth: musicmanager oauth header
+    oauth: musicmanager/mobileclient Bearer header
+    gpsoauth: mobileclient gpsoauth GoogleLogin header
 """
 AuthTypes = namedtuple('AuthTypes', _auth_names)
 
@@ -42,17 +39,17 @@ class BuildRequestMeta(type):
     """Metaclass to create build_request from static/dynamic config."""
 
     def __new__(cls, name, bases, dct):
-        #To not mess with mro and inheritance, build the class first.
-        new_cls = super(BuildRequestMeta, cls).__new__(cls, name, bases, dct)
+        # To not mess with mro and inheritance, build the class first.
+        new_cls = super().__new__(cls, name, bases, dct)
 
         merge_keys = ('headers', 'params')
         all_keys = ('method', 'url', 'files', 'data', 'verify', 'allow_redirects') + merge_keys
 
         config = {}  # stores key: val for static or f(*args, **kwargs) -> val for dyn
-        dyn = lambda key: 'dynamic_' + key
-        stat = lambda key: 'static_' + key
-        has_key = lambda key: hasattr(new_cls, key)
-        get_key = lambda key: getattr(new_cls, key)
+        dyn = lambda key: 'dynamic_' + key  # noqa
+        stat = lambda key: 'static_' + key  # noqa
+        has_key = lambda key: hasattr(new_cls, key)  # noqa
+        get_key = lambda key: getattr(new_cls, key)  # noqa
 
         for key in all_keys:
             if not has_key(dyn(key)) and not has_key(stat(key)):
@@ -64,7 +61,7 @@ class BuildRequestMeta(type):
                 config[key] = get_key(stat(key))
 
         for key in merge_keys:
-            #merge case: dyn took precedence above, but stat also exists
+            # merge case: dyn took precedence above, but stat also exists
             if has_key(dyn(key)) and has_key(stat(key)):
                 def key_closure(stat_val=get_key(stat(key)), dyn_func=get_key(dyn(key))):
                     def build_key(*args, **kwargs):
@@ -75,10 +72,10 @@ class BuildRequestMeta(type):
                     return build_key
                 config[key] = key_closure()
 
-        #To explain some of the funkiness wrt closures, see:
+        # To explain some of the funkiness wrt closures, see:
         # http://stackoverflow.com/questions/233673/lexical-closures-in-python
 
-        #create the actual build_request method
+        # create the actual build_request method
         def req_closure(config=config):
             def build_request(cls, *args, **kwargs):
                 req_kwargs = {}
@@ -96,7 +93,7 @@ class BuildRequestMeta(type):
         return new_cls
 
 
-class Call(object):
+class Call(metaclass=BuildRequestMeta):
     """
     Clients should use Call.perform().
 
@@ -148,9 +145,8 @@ class Call(object):
     Calls are organized semantically, so one endpoint might have multiple calls.
     """
 
-    __metaclass__ = BuildRequestMeta
-
     gets_logged = True
+    fail_on_non_200 = True
 
     required_auth = authtypes()  # all false by default
 
@@ -188,9 +184,10 @@ class Call(object):
         *args and **kwargs are passed to protocol.build_transaction.
 
         :param session: a PlaySession used to send this request.
-        :param validate: if False, do not validate
+        :param validate: if False, do not validate.
+        :param required_auth: if in kwargs, overrides the static protocol required_auth.
         """
-        #TODO link up these docs
+        # TODO link up these docs
 
         call_name = cls.__name__
 
@@ -198,31 +195,33 @@ class Call(object):
             log.debug("%s(args=%s, kwargs=%s)",
                       call_name,
                       [utils.truncate(a) for a in args],
-                      dict((k, utils.truncate(v)) for (k, v) in kwargs.items())
+                      {k: utils.truncate(v) for (k, v) in kwargs.items()}
                       )
         else:
             log.debug("%s(<omitted>)", call_name)
 
+        required_auth = kwargs.pop('required_auth', cls.required_auth)
+
         req_kwargs = cls.build_request(*args, **kwargs)
 
-        response = session.send(req_kwargs, cls.required_auth)
-        #TODO trim the logged response if it's huge?
+        response = session.send(req_kwargs, required_auth)
+        # TODO trim the logged response if it's huge?
 
         safe_req_kwargs = req_kwargs.copy()
         if safe_req_kwargs.get('headers', {}).get('Authorization', None) is not None:
             safe_req_kwargs['headers']['Authorization'] = '<omitted>'
 
-        # check response code
-        try:
-            response.raise_for_status()
-        except requests.HTTPError as e:
-            err_msg = str(e)
+        if cls.fail_on_non_200:
+            try:
+                response.raise_for_status()
+            except requests.HTTPError as e:
+                err_msg = str(e)
 
-            if cls.gets_logged:
-                err_msg += "\n(requests kwargs: %r)" % (safe_req_kwargs)
-                err_msg += "\n(response was: %r)" % response.content
+                if cls.gets_logged:
+                    err_msg += "\n(requests kwargs: %r)" % (safe_req_kwargs)
+                    err_msg += "\n(response was: %r)" % response.text
 
-            raise CallFailure(err_msg, call_name)
+                raise CallFailure(err_msg, call_name)
 
         try:
             parsed_response = cls.parse_response(response)
@@ -231,8 +230,8 @@ class Call(object):
                        " The call may still have succeeded, but it's unlikely.")
             if cls.gets_logged:
                 err_msg += "\n(requests kwargs: %r)" % (safe_req_kwargs)
-                err_msg += "\n(response was: %r)" % response.content
-                log.exception("could not parse %s response: %r", call_name, response.content)
+                err_msg += "\n(response was: %r)" % response.text
+                log.exception("could not parse %s response: %r", call_name, response.text)
             else:
                 log.exception("could not parse %s response: (omitted)", call_name)
 
@@ -242,7 +241,7 @@ class Call(object):
             log.debug(cls.filter_response(parsed_response))
 
         try:
-            #order is important; validate only has a schema for a successful response
+            # order is important; validate only has a schema for a successful response
             cls.check_success(response, parsed_response)
             if validate:
                 cls.validate(response, parsed_response)
@@ -251,25 +250,24 @@ class Call(object):
                 raise
 
             # otherwise, reraise a new exception with our req/res context
-            trace = sys.exc_info()[2]
             err_msg = ("{e_message}\n"
                        "(requests kwargs: {req_kwargs!r})\n"
                        "(response was: {content!r})").format(
-                           e_message=e.message,
+                           e_message=str(e),
                            req_kwargs=safe_req_kwargs,
-                           content=response.content)
-            raise CallFailure(err_msg, e.callname), None, trace
+                           content=response.text)
+            raise CallFailure(err_msg, e.callname) from e
 
         except ValidationException as e:
-            #TODO shouldn't be using formatting
+            # TODO shouldn't be using formatting
             err_msg = "the response format for %s was not recognized." % call_name
             err_msg += "\n\n%s\n" % e
 
             if cls.gets_logged:
-                raw_response = response.content
+                raw_response = response.text
 
-                if len(raw_response) > 1000:
-                    raw_response = raw_response[:1000] + '...'
+                if len(raw_response) > 10000:
+                    raw_response = raw_response[:10000] + '...'
 
                 err_msg += ("\nFirst, try the develop branch."
                             " If you can recreate this error with the most recent code"
@@ -287,8 +285,7 @@ class Call(object):
         try:
             return json.loads(text)
         except ValueError as e:
-            trace = sys.exc_info()[2]
-            raise ParseException(str(e)), None, trace
+            raise ParseException(str(e)) from e
 
     @staticmethod
     def _filter_proto(msg, make_copy=True):
@@ -300,20 +297,20 @@ class Call(object):
 
         fields = filtered.ListFields()
 
-        #eg of filtering a specific field
-        #if any(fd.name == 'field_name' for fd, val in fields):
+        # eg of filtering a specific field
+        # if any(fd.name == 'field_name' for fd, val in fields):
         #    filtered.field_name = '<name>'
 
-        #Filter all byte fields.
+        # Filter all byte fields.
         for field_name, val in ((fd.name, val) for fd, val in fields
                                 if fd.type == FieldDescriptor.TYPE_BYTES):
-            setattr(filtered, field_name, "<%s bytes>" % len(val))
+            setattr(filtered, field_name, bytes("<%s bytes>" % len(val), 'utf8'))
 
-        #Filter submessages.
+        # Filter submessages.
         for field in (val for fd, val in fields
                       if fd.type == FieldDescriptor.TYPE_MESSAGE):
 
-            #protobuf repeated api is bad for reflection
+            # protobuf repeated api is bad for reflection
             is_repeated = hasattr(field, '__len__')
 
             if not is_repeated:
@@ -321,7 +318,7 @@ class Call(object):
 
             else:
                 for i in range(len(field)):
-                    #repeatedComposite does not allow setting
+                    # repeatedComposite does not allow setting
                     old_fields = [f for f in field]
                     del field[:]
 
@@ -329,66 +326,3 @@ class Call(object):
                                   for f in old_fields])
 
         return filtered
-
-
-class ClientLogin(Call):
-    """Performs `Google ClientLogin
-    <https://developers.google.com/accounts/docs/AuthForInstalledApps#ClientLogin>`__."""
-
-    gets_logged = False
-
-    static_method = 'POST'
-    #static_headers = {'User-agent': 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1'}
-    static_url = 'https://www.google.com/accounts/ClientLogin'
-
-    @classmethod
-    def dynamic_data(cls, Email, Passwd, accountType='HOSTED_OR_GOOGLE',
-                     service='sj', source=None,
-                     logintoken=None, logincaptcha=None):
-        """Params align with those in the actual request.
-
-        If *source* is ``None``, ``'gmusicapi-<version>'`` is used.
-
-        Captcha requests are not yet implemented.
-        """
-        if logintoken is not None or logincaptcha is not None:
-            raise ValueError('ClientLogin captcha handling is not yet implemented.')
-
-        if source is None:
-            source = 'gmusicapi-' + gmusicapi.__version__
-
-        return dict(
-            (name, val) for (name, val) in locals().items()
-            if name in set(('Email', 'Passwd', 'accountType', 'service', 'source',
-                            'logintoken', 'logincaptcha'))
-        )
-
-    @classmethod
-    def parse_response(cls, response):
-        """Return a dictionary of response key/vals.
-
-        A successful login will have SID, LSID, and Auth keys.
-        """
-
-        # responses are formatted as, eg:
-        #    SID=DQAAAGgA...7Zg8CTN
-        #    LSID=DQAAAGsA...lk8BBbG
-        #    Auth=DQAAAGgA...dk3fA5N
-        # or:
-        #    Url=http://www.google.com/login/captcha
-        #    Error=CaptchaRequired
-        #    CaptchaToken=DQAAAGgA...dkI1LK9
-        #    CaptchaUrl=Captcha?ctoken=HiteT...
-
-        ret = {}
-        for line in response.text.split('\n'):
-            if '=' in line:
-                var, val = line.split('=', 1)
-                ret[var] = val
-
-        return ret
-
-    @classmethod
-    def check_succes(cls, response, msg):
-        if response.status_code == 200:
-            raise CallFailure("status code %s != 200" % response.status_code, cls.__name__)
