@@ -1,14 +1,12 @@
-# -*- coding: utf-8 -*-
-
 """
 Tests that don't hit the Google Music servers.
 """
 
 from collections import namedtuple
-import time
 import os
+import time
+from unittest.mock import MagicMock
 
-from mock import MagicMock as Mock
 from proboscis.asserts import (
     assert_raises, assert_true, assert_false, assert_equal,
     assert_is_not, Check
@@ -16,8 +14,8 @@ from proboscis.asserts import (
 from proboscis import test
 
 import gmusicapi.session
-from gmusicapi.clients import Webclient, Musicmanager
-from gmusicapi.exceptions import AlreadyLoggedIn  # ,NotLoggedIn
+from gmusicapi.clients import Mobileclient, Musicmanager
+from gmusicapi.exceptions import AlreadyLoggedIn
 from gmusicapi.protocol.shared import authtypes
 from gmusicapi.protocol import mobileclient
 from gmusicapi.utils import utils, jsarray
@@ -27,12 +25,12 @@ jsarray_filenames = [base + '.jsarray' for base in ('searchresult', 'fetchartist
 
 test_file_dir = os.path.dirname(os.path.abspath(__file__))
 for filepath in [os.path.join(test_file_dir, p) for p in jsarray_filenames]:
-    with open(filepath, 'r') as f:
-        jsarray_samples.append(f.read().decode('utf-8'))
+    with open(filepath, 'r', encoding="utf-8") as f:
+        jsarray_samples.append(f.read())
 
-#TODO test gather_local, transcoding
+# TODO test gather_local, transcoding
 
-#All tests end up in the local group.
+# All tests end up in the local group.
 test = test(groups=['local'])
 
 
@@ -40,16 +38,16 @@ test = test(groups=['local'])
 def longest_increasing_sub():
     lisi = utils.longest_increasing_subseq
     assert_equal(lisi([]), [])
-    assert_equal(lisi(range(10, 0, -1)), [1])
-    assert_equal(lisi(range(10, 20)), range(10, 20))
+    assert_equal(lisi(list(range(10, 0, -1))), [1])
+    assert_equal(lisi(list(range(10, 20))), list(range(10, 20)))
     assert_equal(lisi([3, 1, 4, 1, 5, 9, 2, 6, 5, 3, 5, 8, 9, 7, 9]),
                  [1, 2, 3, 5, 8, 9])
 
-##
+#
 # clients
-##
+#
 # this feels like a dumb pattern, but I can't think of a better way
-names = ('Webclient', 'Musicmanager')
+names = ('Mobileclient', 'Musicmanager')  # Webclient removed since testing is disabled.
 Clients = namedtuple('Clients', [n.lower() for n in names])
 
 
@@ -60,7 +58,7 @@ def create_clients():
         c = cls()
 
         # mock out the underlying session
-        c.session = Mock()
+        c.session = MagicMock()
         clients.append(c)
 
     return Clients(*clients)
@@ -68,8 +66,11 @@ def create_clients():
 
 @test
 def no_client_auth_initially():
-    wc = Webclient()
-    assert_false(wc.is_authenticated())
+    # wc = Webclient()
+    # assert_false(wc.is_authenticated())
+
+    mc = Mobileclient()
+    assert_false(mc.is_authenticated())
 
     mm = Musicmanager()
     assert_false(mm.is_authenticated())
@@ -99,9 +100,9 @@ def mm_prevents_bad_mac_format():
 #     assert_equal(api.get_all_playlist_ids(auto=True, user=False),
 #                  {'auto': {}})
 
-##
+#
 # sessions
-##
+#
 Sessions = namedtuple('Sessions', [n.lower() for n in names])
 
 
@@ -112,7 +113,7 @@ def create_sessions():
         s = cls()
 
         # mock out the underlying requests.session
-        s._rsession = Mock()
+        s._rsession = MagicMock()
         sessions.append(s)
 
     return Sessions(*sessions)
@@ -154,7 +155,7 @@ def send_without_auth():
     for s in create_sessions():
         s.is_authenticated = True
 
-        mock_session = Mock()
+        mock_session = MagicMock()
         mock_req_kwargs = {'fake': 'kwargs'}
 
         s.send(mock_req_kwargs, authtypes(), mock_session)
@@ -167,9 +168,10 @@ def send_without_auth():
         mock_session.closed.called_once_with()
 
 
-##
+#
 # protocol
-##
+#
+
 
 @test
 def authtypes_factory_defaults():
@@ -191,12 +193,12 @@ def authtypes_factory_args():
 def mc_url_signing():
     sig, _ = mobileclient.GetStreamUrl.get_signature("Tdr6kq3xznv5kdsphyojox6dtoq",
                                                      "1373247112519")
-    assert_equal(sig, "gua1gInBdaVo7_dSwF9y0kodua0")
+    assert_equal(sig, b"gua1gInBdaVo7_dSwF9y0kodua0")
 
 
-##
+#
 # utils
-##
+#
 
 @test
 def retry_failure_propogation():
@@ -236,3 +238,8 @@ def jsarray_parsing():
     for raw in jsarray_samples:
         # should not raise an exception
         jsarray.loads(raw)
+
+
+@test
+def locate_transcoder():
+    utils.locate_mp3_transcoder()  # should not raise
