@@ -1,13 +1,11 @@
-# -*- coding: utf-8 -*-
-
 """
 Tools to handle Google's ridiculous interchange format.
 """
 
-from StringIO import StringIO
+from io import StringIO
 from tokenize import generate_tokens
 
-from gmusicapi.compat import json
+import json
 
 
 def to_json(s):
@@ -20,7 +18,7 @@ def to_json(s):
     for t in generate_tokens(StringIO(s).readline):
         if out and any(((',' == t[1] == out[-1]),  # double comma
                         (out[-1] == '[' and t[1] == ','),  # comma opening array
-                       )):
+                        )):
             out.append('null')
 
         out.append(t[1])
