@@ -1,12 +1,12 @@
-# -*- coding: utf-8 -*-
-
 """Utility functions used across api code."""
 
+import ast
 from bisect import bisect_left
 from distutils import spawn
 import errno
 import functools
 import inspect
+import itertools
 import logging
 import os
 import re
@@ -19,27 +19,27 @@ from decorator import decorator
 from google.protobuf.descriptor import FieldDescriptor
 
 from gmusicapi import __version__
-from gmusicapi.compat import my_appdirs
-from gmusicapi.exceptions import CallFailure, GmusicapiWarning
+from gmusicapi.appdirs import my_appdirs
+from gmusicapi.exceptions import CallFailure, GmusicapiWarning, NotSubscribed
 
 # this controls the crazy logging setup that checks the callstack;
 #  it should be monkey-patched to False after importing to disable it.
 # when False, static code will simply log in the standard way under the root.
 per_client_logging = True
 
-#Map descriptor.CPPTYPE -> python type.
+# Map descriptor.CPPTYPE -> python type.
 _python_to_cpp_types = {
-    long: ('int32', 'int64', 'uint32', 'uint64'),
+    int: ('int32', 'int64', 'uint32', 'uint64'),
     float: ('double', 'float'),
     bool: ('bool',),
     str: ('string',),
 }
 
-cpp_type_to_python = dict(
-    (getattr(FieldDescriptor, 'CPPTYPE_' + cpp.upper()), python)
+cpp_type_to_python = {
+    getattr(FieldDescriptor, 'CPPTYPE_' + cpp.upper()): python
     for (python, cpplist) in _python_to_cpp_types.items()
     for cpp in cpplist
-)
+}
 
 log_filepath = os.path.join(my_appdirs.user_log_dir, 'gmusicapi.log')
 printed_log_start_message = False  # global, set in config_debug_logging
@@ -49,7 +49,7 @@ printed_log_start_message = False  # global, set in config_debug_logging
 _mac_pattern = re.compile("^({pair}:){{5}}{pair}$".format(pair='[0-9A-F]' * 2))
 
 
-class DynamicClientLogger(object):
+class DynamicClientLogger:
     """Dynamically proxies to the logger of a Client higher in the call stack.
 
     This is a ridiculous hack needed because
@@ -87,10 +87,10 @@ class DynamicClientLogger(object):
                         f_self = frame.f_locals['self']
 
                         # can't import and check against classes; that causes an import cycle
-                        if ((f_self is not None and
-                             f_self.__module__.startswith('gmusicapi.clients') and
-                             f_self.__class__.__name__ in ('Musicmanager', 'Webclient',
-                                                           'Mobileclient'))):
+                        if (f_self is not None and
+                            f_self.__module__.startswith('gmusicapi.clients') and
+                            f_self.__class__.__name__ in ('Musicmanager', 'Webclient',
+                                                          'Mobileclient')):
                             logger = f_self.logger
                             break
                 finally:
@@ -117,7 +117,7 @@ def deprecated(instructions):
 
     @decorator
     def wrapper(func, *args, **kwargs):
-        message = "{0} is deprecated and may break unexpectedly.\n{1}".format(
+        message = "{0} is deprecated and may break unexpectedly; {1}".format(
             func.__name__,
             instructions)
 
@@ -144,10 +144,10 @@ def longest_increasing_subseq(seq):
     # predecessor[j] = linked list of indices of best subsequence ending
     # at seq[j], in reverse order
     predecessor = [-1]
-    for i in xrange(1, len(seq)):
-        ## Find j such that:  seq[head[j - 1]] < seq[i] <= seq[head[j]]
-        ## seq[head[j]] is increasing, so use binary search.
-        j = bisect_left([seq[head[idx]] for idx in xrange(len(head))], seq[i])
+    for i in range(1, len(seq)):
+        # Find j such that:  seq[head[j - 1]] < seq[i] <= seq[head[j]]
+        # seq[head[j]] is increasing, so use binary search.
+        j = bisect_left([seq[head[idx]] for idx in range(len(head))], seq[i])
 
         if j == len(head):
             head.append(i)
@@ -156,7 +156,7 @@ def longest_increasing_subseq(seq):
 
         predecessor.append(head[j - 1] if j > 0 else -1)
 
-    ## trace subsequence back to output
+    # trace subsequence back to output
     result = []
     trace_idx = head[-1]
     while (trace_idx >= 0):
@@ -262,8 +262,8 @@ def dual_decorator(func):
     """
     @functools.wraps(func)
     def inner(*args, **kw):
-        if ((len(args) == 1 and not kw and callable(args[0])
-             and not (type(args[0]) == type and issubclass(args[0], BaseException)))):
+        if (len(args) == 1 and not kw and callable(args[0]) and
+                not (type(args[0]) == type and issubclass(args[0], BaseException))):
             return func()(args[0])
         else:
             return func(*args, **kw)
@@ -281,7 +281,7 @@ def enforce_id_param(position=1):
     @decorator
     def wrapper(function, *args, **kw):
 
-        if not isinstance(args[position], basestring):
+        if not isinstance(args[position], str):
             raise ValueError("Invalid param type in position %s;"
                              " expected an id (did you pass a dictionary?)" % position)
 
@@ -301,8 +301,8 @@ def enforce_ids_param(position=1):
     @decorator
     def wrapper(function, *args, **kw):
 
-        if ((not isinstance(args[position], (list, tuple)) or
-             not all([isinstance(e, basestring) for e in args[position]]))):
+        if (not isinstance(args[position], (list, tuple)) or
+                not all([isinstance(e, str) for e in args[position]])):
             raise ValueError("Invalid param type in position %s;"
                              " expected ids (did you pass dictionaries?)" % position)
 
@@ -322,7 +322,7 @@ def configure_debug_log_handlers(logger):
     logging_to_file = True
     try:
         make_sure_path_exists(os.path.dirname(log_filepath), 0o700)
-        debug_handler = logging.FileHandler(log_filepath)
+        debug_handler = logging.FileHandler(log_filepath, encoding='utf-8')
     except OSError:
         logging_to_file = False
         debug_handler = logging.StreamHandler()
@@ -336,7 +336,7 @@ def configure_debug_log_handlers(logger):
     logger.addHandler(important_handler)
 
     if not printed_log_start_message:
-        #print out startup message without verbose formatting
+        # print out startup message without verbose formatting
         logger.info("!-- begin debug log --!")
         logger.info("version: " + __version__)
         if logging_to_file:
@@ -402,12 +402,12 @@ def pb_set(msg, field_name, val):
     :param val
     """
 
-    #Find the proper type.
+    # Find the proper type.
     field_desc = msg.DESCRIPTOR.fields_by_name[field_name]
     proper_type = cpp_type_to_python[field_desc.cpp_type]
 
-    #Try with the given type first.
-    #Their set hooks will automatically coerce.
+    # Try with the given type first.
+    # Their set hooks will automatically coerce.
     try_types = (type(val), proper_type)
 
     for t in try_types:
@@ -424,6 +424,35 @@ def pb_set(msg, field_name, val):
     return True
 
 
+def locate_mp3_transcoder():
+    """Return the path to a transcoder (ffmpeg or avconv) with mp3 support.
+
+    Raise ValueError if none are suitable."""
+
+    transcoders = ['ffmpeg', 'avconv']
+    transcoder_details = {}
+
+    for transcoder in transcoders:
+        cmd_path = spawn.find_executable(transcoder)
+        if cmd_path is None:
+            transcoder_details[transcoder] = 'not installed'
+            continue
+
+        with open(os.devnull, "w") as null:
+            stdout = subprocess.check_output([cmd_path, '-codecs'], stderr=null).decode("ascii")
+        mp3_encoding_support = ('libmp3lame' in stdout and 'disable-libmp3lame' not in stdout)
+        if mp3_encoding_support:
+            transcoder_details[transcoder] = "mp3 encoding support"
+            break  # mp3 decoding/encoding supported
+        else:
+            transcoder_details[transcoder] = 'no mp3 encoding support'
+    else:
+        raise ValueError('ffmpeg or avconv must be in the path and support mp3 encoding'
+                         "\ndetails: %r" % transcoder_details)
+
+    return cmd_path
+
+
 def transcode_to_mp3(filepath, quality='320k', slice_start=None, slice_duration=None):
     """Return the bytestring result of transcoding the file at *filepath* to mp3.
     An ID3 header is not included in the result.
@@ -434,15 +463,13 @@ def transcode_to_mp3(filepath, quality='320k', slice_start=None, slice_duration=
     :param slice_start: (optional) transcode a slice, starting at this many seconds
     :param slice_duration: (optional) when used with slice_start, the number of seconds in the slice
 
-    Raise IOError on transcoding problems, or ValueError on param problems.
+    Raise:
+      * OSError: problems during transcoding
+      * ValueError: invalid params, transcoder not found
     """
 
     err_output = None
-    cmd_path = spawn.find_executable('ffmpeg')
-    if cmd_path is None:
-        cmd_path = spawn.find_executable('avconv')
-        if cmd_path is None:
-            raise IOError('Neither ffmpeg nor avconv was found in your PATH')
+    cmd_path = locate_mp3_transcoder()
     cmd = [cmd_path, '-i', filepath]
 
     if slice_duration is not None:
@@ -452,7 +479,7 @@ def transcode_to_mp3(filepath, quality='320k', slice_start=None, slice_duration=
 
     if isinstance(quality, int):
         cmd.extend(['-q:a', str(quality)])
-    elif isinstance(quality, basestring):
+    elif isinstance(quality, str):
         cmd.extend(['-b:a', quality])
     else:
         raise ValueError("quality must be int or string, but received %r" % quality)
@@ -470,22 +497,22 @@ def transcode_to_mp3(filepath, quality='320k', slice_start=None, slice_duration=
         audio_out, err_output = proc.communicate()
 
         if proc.returncode != 0:
-            err_output = ("(return code: %r)\n" % proc.returncode) + err_output
-            raise IOError  # handle errors in except
+            err_output = ("(return code: %r)\n" % proc.returncode) + err_output.decode("ascii")
+            raise OSError  # handle errors in except
 
-    except (OSError, IOError) as e:
+    except OSError as e:
 
-        err_msg = "transcoding command (%s) failed: %s. " % (' '.join(cmd), e)
+        err_msg = "transcoding command (%r) failed: %s. " % (' '.join(cmd), e)
 
         if 'No such file or directory' in str(e):
-            err_msg += '\navconv must be installed and in the system path.'
+            err_msg += '\nffmpeg or avconv must be installed and in the system path.'
 
         if err_output is not None:
             err_msg += "\nstderr: '%s'" % err_output
 
         log.exception('transcoding failure:\n%s', err_msg)
 
-        raise IOError(err_msg)
+        raise OSError(err_msg)
 
     else:
         return audio_out
@@ -496,7 +523,7 @@ def truncate(x, max_els=100, recurse_levels=0):
     recurse_levels is only valid for homogeneous lists/tuples.
     max_els ignored for song dictionaries."""
 
-    #Coerce tuple to list to ease truncation.
+    # Coerce tuple to list to ease truncation.
     is_tuple = False
     if isinstance(x, tuple):
         is_tuple = True
@@ -504,17 +531,22 @@ def truncate(x, max_els=100, recurse_levels=0):
 
     try:
         if len(x) > max_els:
-            if isinstance(x, basestring):
+            if isinstance(x, str):
                 return x[:max_els] + '...'
+            elif isinstance(x, bytes):
+                return x[:max_els] + b'...'
 
             if isinstance(x, dict):
                 if 'id' in x and 'titleNorm' in x:
-                    #assume to be a song dict
-                    trunc = dict((k, x.get(k)) for k in ['title', 'artist', 'album'])
+                    # assume to be a song dict
+                    trunc = {k: x.get(k) for k in ['title', 'artist', 'album']}
                     trunc['...'] = '...'
                     return trunc
                 else:
-                    return dict(x.items()[:max_els] + [('...', '...')])
+                    return dict(
+                        itertools.chain(
+                            itertools.islice(x.items(), 0, max_els),
+                            [('...', '...')]))
 
             if isinstance(x, list):
                 trunc = x[:max_els] + ['...']
@@ -525,7 +557,7 @@ def truncate(x, max_els=100, recurse_levels=0):
                 return trunc
 
     except TypeError:
-        #does not have len
+        # does not have len
         pass
 
     return x
@@ -536,25 +568,22 @@ def empty_arg_shortcircuit(return_code='[]', position=1):
     """Decorate a function to shortcircuit and return something immediately if
     the length of a positional arg is 0.
 
-    :param return_code: (optional) code to exec as the return value - default is a list.
+    :param return_code: (optional) simple expression to eval as the return value - default is a list
     :param position: (optional) the position of the expected list - default is 1.
     """
 
-    #The normal pattern when making a collection an optional arg is to use
+    # The normal pattern when making a collection an optional arg is to use
     # a sentinel (like None). Otherwise, you run the risk of the collection
     # being mutated - there's only one, not a new one on each call.
-    #Here we've got multiple things we'd like to
+    # Here we've got multiple things we'd like to
     # return, so we can't do that. Rather than make some kind of enum for
-    # 'accepted return values' I'm just allowing freedom to return anything.
-    #Less safe? Yes. More convenient? Definitely.
+    # 'accepted return values' I'm just allowing freedom to return basic values.
+    # ast.literal_eval only can evaluate most literal expressions (e.g. [] and {})
 
     @decorator
     def wrapper(function, *args, **kw):
         if len(args[position]) == 0:
-            #avoid polluting our namespace
-            ns = {}
-            exec 'retval = ' + return_code in ns
-            return ns['retval']
+            return ast.literal_eval(return_code)
         else:
             return function(*args, **kw)
 
@@ -574,7 +603,7 @@ def accept_singleton(expected_type, position=1):
     def wrapper(function, *args, **kw):
 
         if isinstance(args[position], expected_type):
-            #args are a tuple, can't assign into them
+            # args are a tuple, can't assign into them
             args = list(args)
             args[position] = [args[position]]
             args = tuple(args)
@@ -584,7 +613,68 @@ def accept_singleton(expected_type, position=1):
     return wrapper
 
 
-#Used to mark a field as unimplemented.
+@decorator
+def require_subscription(function, *args, **kwargs):
+    self = args[0]
+
+    if not self.is_subscribed:
+        raise NotSubscribed("%s requires a subscription." % function.__name__)
+
+    return function(*args, **kwargs)
+
+
+# Modification of recipe found at
+# https://wiki.python.org/moin/PythonDecoratorLibrary#Cached_Properties.
+class cached_property:
+    """Version of @property decorator that caches the result with a TTL.
+
+    Tracks the property's value and last refresh time in a dict attribute
+    of a class instance (``self._cache``) using the property name as the key.
+    """
+
+    def __init__(self, ttl=0):
+        self.ttl = ttl
+
+    def __call__(self, fget, doc=None):
+        self.fget = fget
+        self.__doc__ = doc or fget.__doc__
+        self.__name__ = fget.__name__
+        self.__module__ = fget.__module__
+
+        return self
+
+    def __get__(self, inst, owner):
+        now = time.time()
+
+        try:
+            value, last_update = inst._cache[self.__name__]
+
+            if (self.ttl > 0) and (now - last_update > self.ttl):
+                raise AttributeError
+        except (KeyError, AttributeError):
+            value = self.fget(inst)
+
+            try:
+                cache = inst._cache
+            except AttributeError:
+                cache = inst._cache = {}
+
+            cache[self.__name__] = (value, now)
+
+        return value
+
+    def __set__(self, inst, value):
+        raise AttributeError("Can't set cached properties")
+
+    def __delete__(self, inst):
+        try:
+            del inst._cache[self.__name__]
+        except (KeyError, AttributeError):
+            if not inst._cache:
+                inst._cache = {}
+
+
+# Used to mark a field as unimplemented.
 @property
 def NotImplementedField(self):
     raise NotImplementedError
